class ParserException(Exception):
    def __init__(self, *args, **kwargs):
        self.details = kwargs.pop('details', None)
        self.redirect = kwargs.pop('redirect', None)
        self.status = kwargs.pop('status', None)
        super(Exception, self).__init__(*args, **kwargs)



try:
    from io import StringIO
except Exception:
    from StringIO import StringIO

import decimal
import re

from transit.reader import Reader
from transit.writer import Writer
from transit.transit_types import Keyword, frozendict

from django_om import settings


def kebab_to_snake(match):
    return '{0}_{1}'.format(match.group()[0], match.group()[2])


def snake_to_kebab(match):
    return '{0}-{1}'.format(match.group()[0], match.group()[2])


def snakify(data):
    if isinstance(data, frozendict):
        new_dict = {}
        for key, value in data.items():
            key = str(key)  # Convert from transit_types.Keyword
            new_key = re.sub(r'[a-z]-[a-z]', kebab_to_snake, key)
            new_dict[new_key] = snakify(value)
        return new_dict
    if isinstance(data, (list, tuple)):
        data = list(data)
        for i in range(len(data)):
            try:
                data[i] = snakify(data[i])
            except Exception:
                raise Exception(data)
        return data
    if isinstance(data, Keyword):
        data = re.sub(r'[a-z]-[a-z]', kebab_to_snake, str(data))
    return data


def kebabify(data):
    if isinstance(data, dict):
        new_dict = {}
        for key, value in data.items():
            if isinstance(key, str):
                key = re.sub(r"[A-Za-z0-9]_[A-Za-z0-9]", snake_to_kebab, key)
                key = Keyword(key)
            new_dict[key] = kebabify(value)
        return new_dict
    if isinstance(data, (list, tuple)):
        data = list(data)
        for i in range(len(data)):
            data[i] = kebabify(data[i])
        return data
    if isinstance(data, decimal.Decimal):
        return float(data)
    return data


def read_transit(transit_data):
    reader = Reader(settings.ENCODING)
    data = reader.read(StringIO(transit_data))
    data = snakify(data)
    return data


def write_transit(data):
    io = StringIO()
    writer = Writer(io, settings.ENCODING)
    data = kebabify(data)
    writer.write(data)
    transit_data = io.getvalue()
    return transit_data



import importlib
import os

from django.conf import settings

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

DEBUG = settings.DEBUG

SETTINGS = settings.DJANGO_OM

ENCODING = SETTINGS.get('TRANSIT_ENCODING', 'json')
PAGE_SIZE = SETTINGS.get('PAGE_SIZE', 20)

PARSER = importlib.import_module(SETTINGS['PARSER']).PARSER



import os
import re

from distutils.core import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('django_om')

setup(
    name='django-om',
    packages=['django_om'],
    version=version,
    description='Om.next query parser for Django',
    author='Chargeads',
    author_email='devs@chargeads.com',
    url='https://github.com/chargeads/django-om',
    download_url='https://github.com/chargeads/django-om/tarball/0.1',
    keywords=['django', 'om', 'next', 'query', 'parser'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.7',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
